#region Using directives
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Serialization;

using Perception.Sheets;
using Perception;
using Perception.ILO.Engine;
using Perception.Common;
using Perception.Core.ExtensionMethods;
using Perception.Core.Conversion;
using Perception.ILO.Engine.Interfaces;
#endregion

namespace CSI_Acquisition
{
    /// <summary>
    /// The Sheet Control class, implementing the ISheet interface
    /// </summary>
    [SheetInfoRetriever(typeof(SheetControlInfoRetriever))]
    [SheetInformationProvider(typeof(SheetControlInfoProvider))]
    public partial class SheetControl : UserControl, ISheet
    {
        #region Members
        private string m_strUserName = Properties.Resources.IDS_USERNAME;
        private IProgram m_iProgram = null;
        private UIState m_UIState = UIState.Invisible;
        private InitializeState m_InitializeState = InitializeState.Unknown;
        private bool m_bDisposed = false;
        // private string m_strExampleValue = string.Empty;

        #region -> MyMembers

        private CtrlAcquisitionSystem m_MyDemoSystem = null;
        private CtrlGroup m_GroupAll = null;
        private CtrlChannel m_Channel = null;

        #endregion
        #endregion

        #region Constructor
        public SheetControl()
        {
            InitializeComponent();
            Disposed += SheetDisposed;
        }
        #endregion

        #region ISheet Members
        #region -> Events
        /// <summary>
        /// Occurs when the user name of the sheet is changed.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler UserNameChanged;

#pragma warning disable 067
        /// <summary>
        /// Occurs when one or more tool bar items are updated.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler ToolItemsUpdated;

        /// <summary>
        /// Occurs when the sheet asks to rebuild the menu and tool bar.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler RebuildDynamicMenuRequested;
#pragma warning restore 067
        #endregion

        #region -> Properties
        /// <summary>
        /// Gets/Sets the sheet user-name.
        /// </summary>
        public string UserName
        {
            get
            {
                // Return the current UserName
                return this.m_strUserName;
            }
            set
            {
                // Compare the new UserName to the current UserName
                if (String.Compare(this.m_strUserName, value) != 0)
                {
                    // Set the new UserName
                    this.m_strUserName = value;

                    // Fire the UserNameChanged event
                    this.UserNameChanged?.Invoke(this, new EventArgs());
                }
            }
        }

        /// <summary>
        /// Gets the name of the dynamic drop-down menu entry for the sheet.
        /// </summary>
        public string MenuName
        {
            get
            {
                if (String.IsNullOrEmpty(this.m_strUserName))
                {
                    // Return a string from the resource table when no current User Name is available
                    return Properties.Resources.IDS_USERNAME;
                }
                else
                {
                    // Return the current User Name
                    return this.m_strUserName;
                }
            }
        }

        /// <summary>
        /// Gets the icon of the sheet.
        /// </summary>
        public Icon Icon
        {
            get
            {
                return Properties.Resources.SheetIcon;
            }
        }

        /// <summary>
        /// Gets whether the delete may be activated. Application delete should delete the current active item on the sheet.
        /// </summary> 
        public bool DeleteActiveItemSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets/Sets the user-interface state of the sheet.
        /// </summary>
        public UIState UIState
        {
            get
            {
                return this.m_UIState;
            }
            set
            {
                this.m_UIState = value;
            }
        }

        /// <summary>
        /// Gets the initialization state of the sheet.
        /// </summary>
        public InitializeState InitializeState
        {
            get
            {
                return this.m_InitializeState;
            }
        }
        #endregion

        #region -> Methods
        /// <summary>
        /// Update the items in the dynamic drop-down menu.
        /// </summary>
        public void UpdateMenuItems()
        {
        }

        /// <summary>
        /// Delete the actively selected item on the sheet.
        /// </summary>
        public void DeleteActiveItem()
        {
        }

        /// <summary>
        /// Perform steps to auto-configure the sheet.
        /// </summary>
        public void AutoConfig()
        {
        }

        /// <summary>
        /// Initialize the sheet.
        /// </summary>
        /// <param name="iProgram">Interface to the program.</param>
        /// <returns>Returns <b>Succeeded</b> if the initialization is successful.</returns>
        public InitializeState Initialize(IProgram iProgram)
        {
            try
            {
                this.m_iProgram = iProgram;
                this.m_MyDemoSystem = CtrlAcquisitionSystemFactory.Create();
                m_GroupAll = m_MyDemoSystem.Groups.GroupAll;
                HookToGroupAll();
                this.m_InitializeState = InitializeState.Succeeded;
            }
            catch
            {
                this.m_InitializeState = InitializeState.NotAllowed;
                PerceptionMessageBox.Show("Could not initialize sheet", "CSI: Catch", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return this.m_InitializeState;
        }

        private void HookToGroupAll()
        {
            UnHookFromGroupAll();
            if (m_GroupAll != null)
            {
                m_GroupAll.AcquisitionStateChanged += GroupAllAcquisitionStateChanged;
                m_GroupAll.RecorderAdded += GroupAllRecorderAdded;
                m_GroupAll.RecorderRemoved += GroupAllRecorderRemoved;
            }
        }

        private void UnHookFromGroupAll()
        {
            if (m_GroupAll != null)
            {
                try
                {
                    m_GroupAll.AcquisitionStateChanged -= GroupAllAcquisitionStateChanged;
                    m_GroupAll.RecorderAdded -= GroupAllRecorderAdded;
                    m_GroupAll.RecorderRemoved -= GroupAllRecorderRemoved;
                }
                catch
                {
                }
            }
        }

        private void HookToChannelEvent()
        {
            m_Channel.ScCouplingSettingsChanged += ChannelScCouplingChanged;
        }

        private void UnHookChannelEvent()
        {
            if (m_Channel != null)
            {
                try
                {
                    m_Channel.ScCouplingSettingsChanged -= ChannelScCouplingChanged;
                }
                catch
                {
                }
            }
        }

        private void GroupAllRecorderRemoved(object sender, CtrlRecorder Recorder)
        {
            this.InvokeOnUI(() => DoGroupAllRecorderRemoved(sender, Recorder));
        }

        void DoGroupAllRecorderRemoved(object sender, CtrlRecorder Recorder)
        {
            if (m_GroupAll.Recorders.Count != 0)
            {
                return;
            }

            // disable all control related command buttons
            AcqStatus.Text = "****";
            StartCmd.Enabled = false;
            StopCmd.Enabled = false;
            MenuStartCmd.Enabled = false;
            ToolStartCmd.Enabled = false;
            MenuStopCmd.Enabled = false;
            ToolStopCmd.Enabled = false;

            listSignalCoupling.Items.Clear();
            listInputCoupling.Items.Clear();
            listSignalCoupling.Enabled = false;
            listInputCoupling.Enabled = false;
            // Unhook from channel event
            UnHookChannelEvent();
            m_Channel = null;
        }

        private void GroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            this.InvokeOnUI(() => DoGroupAllRecorderAdded(sender, Recorder));
        }

        void DoGroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            Capabilities args;
            // this should be > 0, but you never know
            if (m_GroupAll.Recorders.Count == 0)
                return;

            // now we have at least one recorder, but does it have channels?
            if (m_GroupAll.Recorders[1].Channels.Count == 0)
                return;

            // initialize stuff
            CtrlChannelSignalCoupling CCSC;
            CtrlChannelInputCoupling CCIC;
            string sResult;

            // Connect the object to the physical hardware and hook it to the correct event
            if (m_Channel == null)
            {
                m_Channel = m_GroupAll.Recorders[1].Channels[1];
                HookToChannelEvent();
            }

            // did we already fill this list?
            if (listSignalCoupling.Items.Count == 0)
            {
                // fetch signal coupling capabilities
                m_Channel.GetCapabilities(CtrlChannelCapabilities.CtrlChannelCaps_SignalCoupling, out args);

                // are we allowed to use this setting?
                if (args.SettingUsage != CtrlSettingUsage.CtrlSettingUsage_Selectable)
                    return;

                // is it a list?
                if (args.EntryType == CtrlEntryType.CtrlEntryType_List)
                {
                    // yes to all -> fetch values that we are allowed to use
                    listSignalCoupling.Enabled = true;
                    int vLB = args.ValueListItems.GetLowerBound(0);
                    int vUB = args.ValueListItems.GetUpperBound(0);
                    for (int nItem = vLB; nItem <= vUB; nItem++)
                    {
                        CCSC = (CtrlChannelSignalCoupling)args.ValueListItems.GetValue(nItem);

                        if (CtrlEntryEnabled.CtrlEntryEnabled_Never == (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem))
                            continue;

                        sResult = GetEnumSigConString(CCSC);
                        ComboItem Item = new ComboItem(sResult, (int)CCSC, (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem));
                        listSignalCoupling.Items.Add(Item);
                    }
                    // fetch the actual setting
                    CCSC = m_GroupAll.Recorders[1].Channels[1].SignalCoupling;
                    sResult = GetEnumSigConString(CCSC);
                    // show actual setting
                    listSignalCoupling.SelectedIndex = listSignalCoupling.FindStringExact(sResult);
                }
            }

            // continue with second list if not already done
            if (listInputCoupling.Items.Count == 0)
            {
                // fetch input coupling capabilities
                m_GroupAll.Recorders[1].Channels[1].GetCapabilities(CtrlChannelCapabilities.CtrlChannelCaps_InputCoupling, out args);

                // are we allowed to use this setting?
                if (args.SettingUsage == CtrlSettingUsage.CtrlSettingUsage_NotSupported)
                {
                    return;
                }
                if (args.SettingUsage != CtrlSettingUsage.CtrlSettingUsage_Selectable)
                {
                    // fetch the actual setting
                    CCIC = m_GroupAll.Recorders[1].Channels[1].InputCoupling;
                    sResult = GetEnumInpConString(CCIC);
                    ComboItem Item = new ComboItem(sResult, (int)CCIC, CtrlEntryEnabled.CtrlEntryEnabled_Never);
                    listInputCoupling.Items.Add(Item);
                    // show actual setting
                    listInputCoupling.SelectedIndex = listInputCoupling.FindStringExact(sResult);
                    return;
                }

                // is it a list?
                if (args.EntryType == CtrlEntryType.CtrlEntryType_List)
                {
                    // yes to all -> fetch values that we are allowed to use
                    listInputCoupling.Enabled = true;
                    int vLB = args.ValueListItems.GetLowerBound(0);
                    int vUB = args.ValueListItems.GetUpperBound(0);
                    for (int nItem = vLB; nItem <= vUB; nItem++)
                    {
                        CCIC = (CtrlChannelInputCoupling)args.ValueListItems.GetValue(nItem);

                        if (CtrlEntryEnabled.CtrlEntryEnabled_Never == (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem))
                            continue;

                        sResult = GetEnumInpConString(CCIC);
                        ComboItem Item = new ComboItem(sResult, (int)CCIC, (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem));
                        listInputCoupling.Items.Add(Item);
                    }
                    // fetch the actual setting
                    CCIC = m_GroupAll.Recorders[1].Channels[1].InputCoupling;
                    sResult = GetEnumInpConString(CCIC);
                    // show actual setting
                    listInputCoupling.SelectedIndex = listInputCoupling.FindStringExact(sResult);
                }
            }
        }

        private string GetEnumSigConString(CtrlChannelSignalCoupling sc)
        {
            switch (sc)
            {
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC:
                    return "AC";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_ExternalProbe:
                    return "AC-External";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_Frequency:
                    return "AC-Freq";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_RMS:
                    return "AC-RMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_TrueRMS:
                    return "AC-TrueRMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC:
                    return "DC";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_ExternalProbe:
                    return "DC-External";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_Frequency:
                    return "DC-Freq";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_RMS:
                    return "DC-RMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_TrueRMS:
                    return "DC-TrueRMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_GND:
                    return "GND";
                default:
                    return "----";
            }
        }

        private string GetEnumInpConString(CtrlChannelInputCoupling ic)
        {
            switch (ic)
            {
                case CtrlChannelInputCoupling.CtrlChannelIC_Current:
                    return "Current";
                case CtrlChannelInputCoupling.CtrlChannelIC_Differential:
                    return "Differential";
                case CtrlChannelInputCoupling.CtrlChannelIC_SingleEndedNegative:
                    return "SE Negative";
                case CtrlChannelInputCoupling.CtrlChannelIC_SingleEndedPositive:
                    return "SE Positive";
                case CtrlChannelInputCoupling.CtrlChannelIC_FloatingDifferential:
                    return "Floating Differential";
                default:
                    return "----";
            }
        }

        void ChannelScCouplingChanged(object sender, CtrlCouplingSettingItem Mask, CtrlChannelInputCoupling InCoupling, CtrlChannelSignalCoupling SigCoupling, double MeasuringPeriod, double ReferenceLevel, double Impedance, double InputCapacityLow, double InputCapacityHigh)
        {
            this.InvokeOnUI(() => DoChannelScCouplingChanged(sender, Mask, InCoupling, SigCoupling, MeasuringPeriod, ReferenceLevel, Impedance, InputCapacityLow, InputCapacityHigh));
        }

        private void DoChannelScCouplingChanged(object sender, CtrlCouplingSettingItem Mask, CtrlChannelInputCoupling InCoupling, CtrlChannelSignalCoupling SigCoupling, double MeasuringPeriod, double ReferenceLevel, double Impedance, double InputCapacityLow, double InputCapacityHigh)
        {
            // event handler. we only do the signal coupling changed here
            // although we could also do the input coupling here
            if (Mask != CtrlCouplingSettingItem.CtrlCouplingSettingItem_SignalCoupling)
                return;
            string sResult = GetEnumSigConString(SigCoupling);
            int scIndex = listSignalCoupling.FindStringExact(sResult);
            if (scIndex < 0)
            {
                // item could not be found -> list is incorrect 
                // so rebuild and reset index
                listSignalCoupling.Items.Clear();
                GroupAllRecorderAdded(this, null);
                scIndex = listSignalCoupling.FindStringExact(sResult);
            }
            listSignalCoupling.SelectedIndex = scIndex;
        }

        void GroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            this.InvokeOnUI(() => DoGroupAllAcquisitionStateChanged(sender, Running, OneShot, Stopping, Paused, Idle));
        }

        void DoGroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            if (Running > 0)
            {
                AcqStatus.Text = "Active";
                StartCmd.Enabled = false;
                StopCmd.Enabled = true;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = true;
                ToolStopCmd.Enabled = true;
            }
            else if (Paused > 0)
            {
                AcqStatus.Text = "Paused";
                StartCmd.Enabled = true;
                StopCmd.Enabled = true;
                MenuStartCmd.Enabled = true;
                ToolStartCmd.Enabled = true;
                MenuStopCmd.Enabled = true;
                ToolStopCmd.Enabled = true;
            }
            else if (OneShot > 0)
            {
                AcqStatus.Text = "Single shot mode";
                StartCmd.Enabled = false;
                StopCmd.Enabled = true;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = true;
                ToolStopCmd.Enabled = true;
            }
            else if (Stopping > 0)
            {
                AcqStatus.Text = "Stopping";
                StartCmd.Enabled = false;
                StopCmd.Enabled = false;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = false;
                ToolStopCmd.Enabled = false;
            }
            else if (Idle > 0)
            {
                AcqStatus.Text = "Idle";
                StartCmd.Enabled = true;
                StopCmd.Enabled = false;
                MenuStartCmd.Enabled = true;
                ToolStartCmd.Enabled = true;
                MenuStopCmd.Enabled = false;
                ToolStopCmd.Enabled = false;
            }
            else
            {
                AcqStatus.Text = "****";
                StartCmd.Enabled = false;
                StopCmd.Enabled = false;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = false;
                ToolStopCmd.Enabled = false;
            }

            // Fire the ToolItemsUpdated event to update the toolbar
            this.ToolItemsUpdated?.Invoke(this, new EventArgs());
        }

        /// <summary>
        /// Perform steps that should be taken after a new settings file was loaded.
        /// </summary>
        public void PostLoad()
        {
        }

        /// <summary>
        /// Collect the dynamic drop-down menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic drop-down menu.</returns>
        public ToolStripItem[] GetDynamicMenu()
        {
            ContextMenuStrip strip = this.contextMenuStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        /// <summary>
        /// Collect the dynamic tool bar menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic tool-bar menu.</returns>
        public ToolStripItem[] GetDynamicToolBar()
        {
            ToolStrip strip = this.toolStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        #endregion
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Load data from saved stream (from VWB).
        /// </summary>
        /// <param name="info">Stream from which to load</param>
        public SheetControl(SerializationInfo info, StreamingContext context)
         : this()
        {
            //
            // Example how to restore a persistent value from a stream
            //
            // m_strExampleValue = Tools.GetValue<string>(info, "ExampleValue", m_strExampleValue);
        }

        /// <summary>
        /// Save data into stream when saving VWB
        /// </summary>
        /// <param name="info">Stream in which to save</param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            //
            // Example how to save a persistent value from into a stream
            //
            // info.AddValue("ExampleValue", m_strExampleValue, typeof(string));
        }
        #endregion

        private void SheetControl_Load(object sender, EventArgs e)
        {
            AcqStatus.Text = "****";
            StartCmd.Enabled = false;
            StopCmd.Enabled = false;
            MenuStartCmd.Enabled = false;
            ToolStartCmd.Enabled = false;
            MenuStopCmd.Enabled = false;
            ToolStopCmd.Enabled = false;
            listSignalCoupling.Enabled = false;
            listInputCoupling.Enabled = false;
        }

        private void StartCmd_Click(object sender, EventArgs e)
        {
            // for demo purposes -> set some acquisition parameters here
            // High Sampling rate to 10kHz; Pre and Post trigger duration to 50ms
            foreach (CtrlGroup myGroup in m_MyDemoSystem.Groups)
            {
                myGroup.HighSamplingFrequency = 10_000;
            }

            IAcquisitionSetup setup = m_MyDemoSystem.AcquisitionSetup;
            setup.PostTriggerInSeconds = 0.05;
            setup.PreTriggerInSeconds = 0.05;

            m_GroupAll.Run();
            StartCmd.Enabled = false;
            StopCmd.Enabled = true;
        }

        private void StopCmd_Click(object sender, EventArgs e)
        {
            m_GroupAll.Stop();
            StartCmd.Enabled = true;
            StopCmd.Enabled = false;
        }

        private void btnGetSpan_Click(object sender, EventArgs e)
        {
            // Check if there are recorders available
            if (m_GroupAll.Recorders.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Check if the first recorder contains channels
            if (m_GroupAll.Recorders[1].Channels.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CtrlChannel myChannel = m_GroupAll.Recorders[1].Channels[1];
            TextBoxSpan.Text = myChannel.Span.ToString();
        }

        private void btnSetSpan_Click(object sender, EventArgs e)
        {
            // Check if there are recorders available
            if (m_GroupAll.Recorders.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Check if the first recorder contains channels
            if (m_GroupAll.Recorders[1].Channels.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CtrlChannel myChannel = m_GroupAll.Recorders[1].Channels[1];

            double dSpan = FloatingPoint.ConvertStringToDouble(TextBoxSpan.Text);
            // Try to convert the text box string input into a double
            if (double.IsNaN(dSpan))
            {
                string cMsg = string.Format("Error setting the Span of the channel: {0} to {1}",
                    myChannel.Name, TextBoxSpan.Text);
                PerceptionMessageBox.Show(this, cMsg, "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else
            {
                myChannel.Span = dSpan;
            }

            // Read back the Span and show it in the text box
            TextBoxSpan.Text = myChannel.Span.ToString();
        }

        private void listSignalCoupling_SelectionChangeCommitted(object sender, EventArgs e)
        {
            ComboItem Item = listSignalCoupling.SelectedItem as ComboItem;

            if (Item.m_Enabled == CtrlEntryEnabled.CtrlEntryEnabled_WhenIdle)
            {
                // Check acquisition status
                int Live;
                int Pause;
                int HoldNext;
                int HoldLast;
                int Idle;

                m_GroupAll.GetAcquisitionState(out Live, out HoldNext, out HoldLast, out Pause, out Idle);
                if (Idle != m_GroupAll.Recorders.Count)
                {
                    listSignalCoupling.SelectedText = GetEnumSigConString(m_Channel.SignalCoupling);
                    return;
                }
            }
            m_Channel.SignalCoupling = (CtrlChannelSignalCoupling)Item.m_Value;
        }

        private void listSignalCoupling_DropDown(object sender, EventArgs e)
        {
            listSignalCoupling.Items.Clear();
            GroupAllRecorderAdded(this, null);
        }

        #region Disposing
        /// <summary>
        /// Procedure to do your cleanup, this function is called from within the Dispose() method.
        /// </summary>
        private void SheetDisposed(object sender, EventArgs e)
        {
            if (IsDisposed)
                return;

            if (m_bDisposed)
                return;
            try
            {
                // Add your clean up code
                if (m_Channel != null)
                {
                    UnHookChannelEvent();
                    m_Channel = null;
                }
                UnHookFromGroupAll();
                m_GroupAll = null;
                m_MyDemoSystem = null;
            }
            catch
            {
            }
            m_bDisposed = true;
        }
        #endregion
    }
}