#region Using directives
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Serialization;

using Perception.Sheets;
using Perception;
using Perception.ILO.Engine;
using Perception.Common;
using Perception.Core.ExtensionMethods;
using Perception.Core.Conversion;
using DataSrcManager;
using RecordingInterface;
using Perception.UserKeys;
using Perception.UserVariables;
using TimeView;
using Perception.WavCalc;
using Perception.FormulaDatabase;
using Perception.ILO.Engine.Interfaces;
#endregion

namespace CSI_Acquisition
{
    /// <summary>
    /// The Sheet Control class, implementing the ISheet interface
    /// </summary>
    [SheetInfoRetriever(typeof(SheetControlInfoRetriever))]
    [SheetInformationProvider(typeof(SheetControlInfoProvider))]
    public partial class SheetControl : UserControl, ISheet
    {
        #region Members
        private string m_strUserName = Properties.Resources.IDS_USERNAME;
        private IProgram m_iProgram = null;
        private UIState m_UIState = UIState.Invisible;
        private InitializeState m_InitializeState = InitializeState.Unknown;
        private bool m_bDisposed = false;
        // private string m_strExampleValue = string.Empty;

        #region -> MyMembers

        private CtrlAcquisitionSystem m_MyDemoSystem = null;
        private CtrlGroup m_GroupAll = null;
        private CtrlChannel m_Channel = null;
        private DataManager m_DataManager = null;
        private PoolEntry m_ActiveCursor = null;
        private IDataSrc m_MySource = null;

        #endregion
        #endregion

        #region Constructor
        public SheetControl()
        {
            InitializeComponent();
            if (m_DataManager == null)
            {
                m_DataManager = new DataManager();
            }
            if (m_DataManager != null)
            {
                m_ActiveCursor = m_DataManager.PoolEntries["Display.CSIDisplay.ActiveCursor.XPosition"];
            }

            HookToActiveCursor();

            Disposed += SheetDisposed;
        }
        #endregion

        #region ISheet Members
        #region -> Events
        /// <summary>
        /// Occurs when the user name of the sheet is changed.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler UserNameChanged;

#pragma warning disable 067
        /// <summary>
        /// Occurs when one or more tool bar items are updated.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler ToolItemsUpdated;

        /// <summary>
        /// Occurs when the sheet asks to rebuild the menu and tool bar.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler RebuildDynamicMenuRequested;
#pragma warning restore 067
        #endregion

        #region -> Properties
        /// <summary>
        /// Gets/Sets the sheet user-name.
        /// </summary>
        public string UserName
        {
            get
            {
                // Return the current UserName
                return this.m_strUserName;
            }
            set
            {
                // Compare the new UserName to the current UserName
                if (String.Compare(this.m_strUserName, value) != 0)
                {
                    // Set the new UserName
                    this.m_strUserName = value;

                    // Fire the UserNameChanged event
                    this.UserNameChanged?.Invoke(this, new EventArgs());
                }
            }
        }

        /// <summary>
        /// Gets the name of the dynamic drop-down menu entry for the sheet.
        /// </summary>
        public string MenuName
        {
            get
            {
                if (String.IsNullOrEmpty(this.m_strUserName))
                {
                    // Return a string from the resource table when no current User Name is available
                    return Properties.Resources.IDS_USERNAME;
                }
                else
                {
                    // Return the current User Name
                    return this.m_strUserName;
                }
            }
        }

        /// <summary>
        /// Gets the icon of the sheet.
        /// </summary>
        public Icon Icon
        {
            get
            {
                return Properties.Resources.SheetIcon;
            }
        }

        /// <summary>
        /// Gets whether the delete may be activated. Application delete should delete the current active item on the sheet.
        /// </summary> 
        public bool DeleteActiveItemSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets/Sets the user-interface state of the sheet.
        /// </summary>
        public UIState UIState
        {
            get
            {
                return this.m_UIState;
            }
            set
            {
                this.m_UIState = value;
            }
        }

        /// <summary>
        /// Gets the initialization state of the sheet.
        /// </summary>
        public InitializeState InitializeState
        {
            get
            {
                return this.m_InitializeState;
            }
        }
        #endregion

        #region -> Methods
        /// <summary>
        /// Update the items in the dynamic drop-down menu.
        /// </summary>
        public void UpdateMenuItems()
        {
        }

        /// <summary>
        /// Delete the actively selected item on the sheet.
        /// </summary>
        public void DeleteActiveItem()
        {
        }

        /// <summary>
        /// Perform steps to auto-configure the sheet.
        /// </summary>
        public void AutoConfig()
        {
        }

        /// <summary>
        /// Initialize the sheet.
        /// </summary>
        /// <param name="iProgram">Interface to the program.</param>
        /// <returns>Returns <b>Succeeded</b> if the initialization is successful.</returns>
        public InitializeState Initialize(IProgram iProgram)
        {
            try
            {
                this.m_iProgram = iProgram;
                this.m_MyDemoSystem = CtrlAcquisitionSystemFactory.Create();
                m_GroupAll = m_MyDemoSystem.Groups.GroupAll;
                HookToGroupAll();
                this.m_InitializeState = InitializeState.Succeeded;
            }
            catch
            {
                this.m_InitializeState = InitializeState.NotAllowed;
                PerceptionMessageBox.Show("Could not initialize sheet", "CSI: Catch", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return this.m_InitializeState;
        }

        private void HookToGroupAll()
        {
            UnHookFromGroupAll();
            if (m_GroupAll != null)
            {
                m_GroupAll.AcquisitionStateChanged += GroupAllAcquisitionStateChanged;
                m_GroupAll.RecorderAdded += GroupAllRecorderAdded;
                m_GroupAll.RecorderRemoved += GroupAllRecorderRemoved;
            }
        }

        private void UnHookFromGroupAll()
        {
            if (m_GroupAll != null)
            {
                try
                {
                    m_GroupAll.AcquisitionStateChanged -= GroupAllAcquisitionStateChanged;
                    m_GroupAll.RecorderAdded -= GroupAllRecorderAdded;
                    m_GroupAll.RecorderRemoved -= GroupAllRecorderRemoved;
                }
                catch
                {
                }
            }
        }

        private void HookToChannelEvent()
        {
            m_Channel.ScCouplingSettingsChanged += ChannelScCouplingChanged;
        }

        private void UnHookChannelEvent()
        {
            if (m_Channel != null)
            {
                try
                {
                    m_Channel.ScCouplingSettingsChanged -= ChannelScCouplingChanged;
                }
                catch
                {
                }
            }
        }

        private void GroupAllRecorderRemoved(object sender, CtrlRecorder Recorder)
        {
            this.InvokeOnUI(() => DoGroupAllRecorderRemoved(sender, Recorder));
        }

        void DoGroupAllRecorderRemoved(object sender, CtrlRecorder Recorder)
        {
            if (m_GroupAll.Recorders.Count != 0)
            {
                return;
            }

            // disable all control related command buttons
            AcqStatus.Text = "****";
            StartCmd.Enabled = false;
            StopCmd.Enabled = false;
            MenuStartCmd.Enabled = false;
            ToolStartCmd.Enabled = false;
            MenuStopCmd.Enabled = false;
            ToolStopCmd.Enabled = false;

            listSignalCoupling.Items.Clear();
            listInputCoupling.Items.Clear();
            listSignalCoupling.Enabled = false;
            listInputCoupling.Enabled = false;
            // Unhook from channel event
            UnHookChannelEvent();
            m_Channel = null;
        }

        private void GroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            this.InvokeOnUI(() => DoGroupAllRecorderAdded(sender, Recorder));
        }

        void DoGroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            Capabilities args;
            // this should be > 0, but you never know
            if (m_GroupAll.Recorders.Count == 0)
                return;

            // now we have at least one recorder, but does it have channels?
            if (m_GroupAll.Recorders[1].Channels.Count == 0)
                return;

            // initialize stuff
            CtrlChannelSignalCoupling CCSC;
            CtrlChannelInputCoupling CCIC;
            string sResult;

            // Connect the object to the physical hardware and hook it to the correct event
            if (m_Channel == null)
            {
                m_Channel = m_GroupAll.Recorders[1].Channels[1];
                HookToChannelEvent();
            }

            // did we already fill this list?
            if (listSignalCoupling.Items.Count == 0)
            {
                // fetch signal coupling capabilities
                m_Channel.GetCapabilities(CtrlChannelCapabilities.CtrlChannelCaps_SignalCoupling, out args);

                // are we allowed to use this setting?
                if (args.SettingUsage != CtrlSettingUsage.CtrlSettingUsage_Selectable)
                    return;

                // is it a list?
                if (args.EntryType == CtrlEntryType.CtrlEntryType_List)
                {
                    // yes to all -> fetch values that we are allowed to use
                    listSignalCoupling.Enabled = true;
                    int vLB = args.ValueListItems.GetLowerBound(0);
                    int vUB = args.ValueListItems.GetUpperBound(0);
                    for (int nItem = vLB; nItem <= vUB; nItem++)
                    {
                        CCSC = (CtrlChannelSignalCoupling)args.ValueListItems.GetValue(nItem);

                        if (CtrlEntryEnabled.CtrlEntryEnabled_Never == (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem))
                            continue;

                        sResult = GetEnumSigConString(CCSC);
                        ComboItem Item = new ComboItem(sResult, (int)CCSC, (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem));
                        listSignalCoupling.Items.Add(Item);
                    }
                    // fetch the actual setting
                    CCSC = m_GroupAll.Recorders[1].Channels[1].SignalCoupling;
                    sResult = GetEnumSigConString(CCSC);
                    // show actual setting
                    listSignalCoupling.SelectedIndex = listSignalCoupling.FindStringExact(sResult);
                }
            }

            // continue with second list if not already done
            if (listInputCoupling.Items.Count == 0)
            {
                // fetch input coupling capabilities
                m_GroupAll.Recorders[1].Channels[1].GetCapabilities(CtrlChannelCapabilities.CtrlChannelCaps_InputCoupling, out args);

                // are we allowed to use this setting?
                if (args.SettingUsage == CtrlSettingUsage.CtrlSettingUsage_NotSupported)
                {
                    return;
                }
                if (args.SettingUsage != CtrlSettingUsage.CtrlSettingUsage_Selectable)
                {
                    // fetch the actual setting
                    CCIC = m_GroupAll.Recorders[1].Channels[1].InputCoupling;
                    sResult = GetEnumInpConString(CCIC);
                    ComboItem Item = new ComboItem(sResult, (int)CCIC, CtrlEntryEnabled.CtrlEntryEnabled_Never);
                    listInputCoupling.Items.Add(Item);
                    // show actual setting
                    listInputCoupling.SelectedIndex = listInputCoupling.FindStringExact(sResult);
                    return;
                }

                // is it a list?
                if (args.EntryType == CtrlEntryType.CtrlEntryType_List)
                {
                    // yes to all -> fetch values that we are allowed to use
                    listInputCoupling.Enabled = true;
                    int vLB = args.ValueListItems.GetLowerBound(0);
                    int vUB = args.ValueListItems.GetUpperBound(0);
                    for (int nItem = vLB; nItem <= vUB; nItem++)
                    {
                        CCIC = (CtrlChannelInputCoupling)args.ValueListItems.GetValue(nItem);

                        if (CtrlEntryEnabled.CtrlEntryEnabled_Never == (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem))
                            continue;

                        sResult = GetEnumInpConString(CCIC);
                        ComboItem Item = new ComboItem(sResult, (int)CCIC, (CtrlEntryEnabled)args.ValueListItemsEnabled.GetValue(nItem));
                        listInputCoupling.Items.Add(Item);
                    }
                    // fetch the actual setting
                    CCIC = m_GroupAll.Recorders[1].Channels[1].InputCoupling;
                    sResult = GetEnumInpConString(CCIC);
                    // show actual setting
                    listInputCoupling.SelectedIndex = listInputCoupling.FindStringExact(sResult);
                }
            }
        }

        private string GetEnumSigConString(CtrlChannelSignalCoupling sc)
        {
            switch (sc)
            {
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC:
                    return "AC";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_ExternalProbe:
                    return "AC-External";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_Frequency:
                    return "AC-Freq";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_RMS:
                    return "AC-RMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_AC_TrueRMS:
                    return "AC-TrueRMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC:
                    return "DC";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_ExternalProbe:
                    return "DC-External";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_Frequency:
                    return "DC-Freq";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_RMS:
                    return "DC-RMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_DC_TrueRMS:
                    return "DC-TrueRMS";
                case CtrlChannelSignalCoupling.CtrlChannelSC_GND:
                    return "GND";
                default:
                    return "----";
            }
        }

        private string GetEnumInpConString(CtrlChannelInputCoupling ic)
        {
            switch (ic)
            {
                case CtrlChannelInputCoupling.CtrlChannelIC_Current:
                    return "Current";
                case CtrlChannelInputCoupling.CtrlChannelIC_Differential:
                    return "Differential";
                case CtrlChannelInputCoupling.CtrlChannelIC_SingleEndedNegative:
                    return "SE Negative";
                case CtrlChannelInputCoupling.CtrlChannelIC_SingleEndedPositive:
                    return "SE Positive";
                case CtrlChannelInputCoupling.CtrlChannelIC_FloatingDifferential:
                    return "Floating Differential";
                default:
                    return "----";
            }
        }

        void ChannelScCouplingChanged(object sender, CtrlCouplingSettingItem Mask, CtrlChannelInputCoupling InCoupling, CtrlChannelSignalCoupling SigCoupling, double MeasuringPeriod, double ReferenceLevel, double Impedance, double InputCapacityLow, double InputCapacityHigh)
        {
            this.InvokeOnUI(() => DoChannelScCouplingChanged(sender, Mask, InCoupling, SigCoupling, MeasuringPeriod, ReferenceLevel, Impedance, InputCapacityLow, InputCapacityHigh));
        }

        private void DoChannelScCouplingChanged(object sender, CtrlCouplingSettingItem Mask, CtrlChannelInputCoupling InCoupling, CtrlChannelSignalCoupling SigCoupling, double MeasuringPeriod, double ReferenceLevel, double Impedance, double InputCapacityLow, double InputCapacityHigh)
        {
            // event handler. we only do the signal coupling changed here
            // although we could also do the input coupling here
            if (Mask != CtrlCouplingSettingItem.CtrlCouplingSettingItem_SignalCoupling)
                return;
            string sResult = GetEnumSigConString(SigCoupling);
            int scIndex = listSignalCoupling.FindStringExact(sResult);
            if (scIndex < 0)
            {
                // item could not be found -> list is incorrect 
                // so rebuild and reset index
                listSignalCoupling.Items.Clear();
                GroupAllRecorderAdded(this, null);
                scIndex = listSignalCoupling.FindStringExact(sResult);
            }
            listSignalCoupling.SelectedIndex = scIndex;
        }

        void GroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            this.InvokeOnUI(() => DoGroupAllAcquisitionStateChanged(sender, Running, OneShot, Stopping, Paused, Idle));
        }

        void DoGroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            if (Running > 0)
            {
                AcqStatus.Text = "Active";
                StartCmd.Enabled = false;
                StopCmd.Enabled = true;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = true;
                ToolStopCmd.Enabled = true;
                display1.pDisplay.AcquisitionView = TimeView.AcquisitionView.AcquisitionView_AddLive;
                display1.pDisplay.TimeDisplay.AcquisitionState = TimeView.AcquisitionState.AcquisitionState_Run;
            }
            else if (Paused > 0)
            {
                AcqStatus.Text = "Paused";
                StartCmd.Enabled = true;
                StopCmd.Enabled = true;
                MenuStartCmd.Enabled = true;
                ToolStartCmd.Enabled = true;
                MenuStopCmd.Enabled = true;
                ToolStopCmd.Enabled = true;
            }
            else if (OneShot > 0)
            {
                AcqStatus.Text = "Single shot mode";
                StartCmd.Enabled = false;
                StopCmd.Enabled = true;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = true;
                ToolStopCmd.Enabled = true;
            }
            else if (Stopping > 0)
            {
                AcqStatus.Text = "Stopping";
                StartCmd.Enabled = false;
                StopCmd.Enabled = false;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = false;
                ToolStopCmd.Enabled = false;
            }
            else if (Idle > 0)
            {
                AcqStatus.Text = "Idle";
                StartCmd.Enabled = true;
                StopCmd.Enabled = false;
                MenuStartCmd.Enabled = true;
                ToolStartCmd.Enabled = true;
                MenuStopCmd.Enabled = false;
                ToolStopCmd.Enabled = false;
                display1.pDisplay.ReviewType = TimeView.ReviewModeType.ReviewModeType_Recording;
                display1.pDisplay.TimeDisplay.AcquisitionState = TimeView.AcquisitionState.AcquisitionState_Idle;
            }
            else
            {
                AcqStatus.Text = "****";
                StartCmd.Enabled = false;
                StopCmd.Enabled = false;
                MenuStartCmd.Enabled = false;
                ToolStartCmd.Enabled = false;
                MenuStopCmd.Enabled = false;
                ToolStopCmd.Enabled = false;
            }

            // Fire the ToolItemsUpdated event to update the toolbar
            this.ToolItemsUpdated?.Invoke(this, new EventArgs());
        }

        /// <summary>
        /// Perform steps that should be taken after a new settings file was loaded.
        /// </summary>
        public void PostLoad()
        {
        }

        /// <summary>
        /// Collect the dynamic drop-down menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic drop-down menu.</returns>
        public ToolStripItem[] GetDynamicMenu()
        {
            ContextMenuStrip strip = this.contextMenuStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        /// <summary>
        /// Collect the dynamic tool bar menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic tool-bar menu.</returns>
        public ToolStripItem[] GetDynamicToolBar()
        {
            ToolStrip strip = this.toolStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        #endregion
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Load data from saved stream (from VWB).
        /// </summary>
        /// <param name="info">Stream from which to load</param>
        public SheetControl(SerializationInfo info, StreamingContext context)
         : this()
        {
            //
            // Example how to restore a persistent value from a stream
            //
            // m_strExampleValue = Tools.GetValue<string>(info, "ExampleValue", m_strExampleValue);
        }

        /// <summary>
        /// Save data into stream when saving VWB
        /// </summary>
        /// <param name="info">Stream in which to save</param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            //
            // Example how to save a persistent value from into a stream
            //
            // info.AddValue("ExampleValue", m_strExampleValue, typeof(string));
        }
        #endregion

        private void SheetControl_Load(object sender, EventArgs e)
        {
            AcqStatus.Text = "****";
            StartCmd.Enabled = false;
            StopCmd.Enabled = false;
            FillCmd.Enabled = true;
            MenuStartCmd.Enabled = false;
            ToolStartCmd.Enabled = false;
            MenuStopCmd.Enabled = false;
            ToolStopCmd.Enabled = false;
            listSignalCoupling.Enabled = false;
            listInputCoupling.Enabled = false;
            display1.UserName = "CSIDisplay";
            // set meter name and connect meter to the correct pool entry
            meter1.UserName = "CSIMeter";
            string[] aPoolEntry = new string[] { "Display.CSIDisplay.ActiveCursor.YValue" };
            meter1.pMeter.LDSMeter.AddDataSources(aPoolEntry);
        }

        private void StartCmd_Click(object sender, EventArgs e)
        {
            // for demo purposes -> set some acquisition parameters here
            // High Sampling rate to 10kHz; Pre and Post trigger duration to 50ms
            foreach (CtrlGroup myGroup in m_MyDemoSystem.Groups)
            {
                myGroup.HighSamplingFrequency = 10_000;
                }

            IAcquisitionSetup setup = m_MyDemoSystem.AcquisitionSetup;
            setup.PostTriggerInSeconds = 0.05;
            setup.PreTriggerInSeconds = 0.05;

            m_GroupAll.Run();
            StartCmd.Enabled = false;
            StopCmd.Enabled = true;
        }

        private void StopCmd_Click(object sender, EventArgs e)
        {
            m_GroupAll.Stop();
            StartCmd.Enabled = true;
            StopCmd.Enabled = false;
        }

        private void btnGetSpan_Click(object sender, EventArgs e)
        {
            // Check if there are recorders available
            if (m_GroupAll.Recorders.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Check if the first recorder contains channels
            if (m_GroupAll.Recorders[1].Channels.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CtrlChannel myChannel = m_GroupAll.Recorders[1].Channels[1];
            TextBoxSpan.Text = myChannel.Span.ToString();
        }

        private void btnSetSpan_Click(object sender, EventArgs e)
        {
            // Check if there are recorders available
            if (m_GroupAll.Recorders.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Check if the first recorder contains channels
            if (m_GroupAll.Recorders[1].Channels.Count < 1)
            {
                PerceptionMessageBox.Show(this, "No Channels connected", "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            CtrlChannel myChannel = m_GroupAll.Recorders[1].Channels[1];

            double dSpan = FloatingPoint.ConvertStringToDouble(TextBoxSpan.Text);
            // Try to convert the text box string input into a double
            if (double.IsNaN(dSpan))
            {
                string cMsg = string.Format("Error setting the Span of the channel: {0} to {1}",
                    myChannel.Name, TextBoxSpan.Text);
                PerceptionMessageBox.Show(this, cMsg, "CSI: Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else
            {
                myChannel.Span = dSpan;
            }

            // Read back the Span and show it in the text box
            TextBoxSpan.Text = myChannel.Span.ToString();
        }

        private void listSignalCoupling_SelectionChangeCommitted(object sender, EventArgs e)
        {
            ComboItem Item = listSignalCoupling.SelectedItem as ComboItem;

            if (Item.m_Enabled == CtrlEntryEnabled.CtrlEntryEnabled_WhenIdle)
            {
                // Check acquisition status
                int Live;
                int Pause;
                int HoldNext;
                int HoldLast;
                int Idle;

                m_GroupAll.GetAcquisitionState(out Live, out HoldNext, out HoldLast, out Pause, out Idle);
                if (Idle != m_GroupAll.Recorders.Count)
                {
                    listSignalCoupling.SelectedText = GetEnumSigConString(m_Channel.SignalCoupling);
                    return;
                }
            }
            m_Channel.SignalCoupling = (CtrlChannelSignalCoupling)Item.m_Value;
        }

        private void listSignalCoupling_DropDown(object sender, EventArgs e)
        {
            listSignalCoupling.Items.Clear();
            GroupAllRecorderAdded(this, null);
        }
        private void FillCmd_Click(object sender, EventArgs e)
        {
            // Clear the dsiplay
            DisplayHelper.ClearDisplay(display1.pDisplay);
        }

        private void HookToActiveCursor()
        {
            UnHookFromActiveCursor();
            if (m_ActiveCursor != null)
            {
                m_ActiveCursor.DataChanged += ActiveCursor_DataChanged;
            }
        }

        private void UnHookFromActiveCursor()
        {
            if (m_ActiveCursor != null)
            {
                m_ActiveCursor.DataChanged -= ActiveCursor_DataChanged;
            }
        }

        void ActiveCursor_DataChanged()
        {
            this.InvokeOnUI(() =>
            {
                if (m_ActiveCursor != null)
                {
                    double dXVal = (double)m_ActiveCursor.DataSource.Value;
                    XValueLbl.Text = dXVal.ToString("##0.000");
                }
            });
        }

        private void AnalysisCmd_Click(object sender, EventArgs e)
        {
            // start with defining the area of interest:
            // +/- 20% of display range around active cursor

            // get active cursor time position
            double dActTime = display1.pDisplay.TimeDisplay.Cursors.ActiveItem.time;

            // center display around this time
            display1.pDisplay.TimeDisplay.CtlLayout.TimeController.CentralTime = dActTime;

            // fetch start and endtime = range
            double dStartTime = display1.pDisplay.TimeDisplay.CtlLayout.TimeController.StartTime;
            double dEndTime = display1.pDisplay.TimeDisplay.CtlLayout.TimeController.EndTime;
            double dRange = dEndTime - dStartTime;

            // set interval to +/- 20% of range around active cursor time
            double dStep = 0.2 * dRange;

            // set cursors
            dStartTime = dActTime - dStep;
            dEndTime = dActTime + dStep;
            display1.pDisplay.TimeDisplay.Cursors[1].time = dStartTime;
            display1.pDisplay.TimeDisplay.Cursors[2].time = dEndTime;
            display1.pDisplay.TimeDisplay.CtlLayout.HorizontalCursors.Visible = true;
            display1.pDisplay.TimeDisplay.CtlLayout.HorizontalCursors[1].Location = (double)0.0;

            // locate data source of active trace
            string sActDataSource = display1.pDisplay.TimeDisplay.ActiveTrace.TraceProp.DataSourceName;
            if (sActDataSource == null)
            {
                PerceptionMessageBox.Show(this, "Can not proceed, there is no active trace.\n\r" +
                     "Open a recording and drag a trace into the display.",
                     "CSI-Data Analysis - Part One", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // use only analog waveforms for the this example
            m_MySource = m_DataManager.PoolEntries[sActDataSource].DataSource;
            if (m_MySource.DataType != DataSourceDataType.DataSourceDataType_AnalogWaveform)
            {
                PerceptionMessageBox.Show(this, "Can not proceed, the active trace has a wrong type.\n\r" +
                     "The type is: '" + m_MySource.DataType.ToString() + "'",
                     "CSI-Data Analysis - Part One", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // initialize variables
            object Result;
            IDataSegments Segments;

            // fetch data
            m_MySource.Data(dStartTime, dEndTime, out Result);

            // fetch segments within this data
            Segments = Result as IDataSegments;
            SegmentsCnt.Text = Segments.Count.ToString();

            // no segments
            if (Segments.Count == 0)
                return;

            // initialize waveform stuff for the user data sources
            WaveformDataBlock Block = null;
            WaveformDataSource MyWave = null;

            // initialize also for calculated average value
            WaveformDataBlock BlockAve = null;
            WaveformDataSource MyWaveAve = null;

            UserDataSources MySources = UserDataSources.Instance;
            MySources.RemoveDatasource(MySources["CSIDemo.MyResult"]);
            MyWave = MySources.CreateWaveForm("CSIDemo.MyResult", "My Waveform");

            // create waveform also for calculated average value
            MySources.RemoveDatasource(MySources["CSIDemo.MyResultAve"]);
            MyWaveAve = MySources.CreateWaveForm("CSIDemo.MyResultAve", "My Average Waveform");

            // set initial parameters
            MyWave.XUnits = m_MySource.XUnit;
            MyWave.YUnits = m_MySource.YUnit;
            MyWave.DisplayRangeFrom = Segments[1].DisplayFrom;
            MyWave.DisplayRangeTo = Segments[1].DisplayTo;

            // set initial parameters also for average waveform
            MyWaveAve.XUnits = m_MySource.XUnit;
            MyWaveAve.YUnits = m_MySource.YUnit;
            MyWaveAve.DisplayRangeFrom = Segments[1].DisplayFrom;
            MyWaveAve.DisplayRangeTo = Segments[1].DisplayTo;

            // loop through all segments to find out the average value
            double dAverage = 0;
            int iTotalCnt = 0;

            // for all segments
            for (int j = 1; j <= Segments.Count; j++)
            {
                // fetch the data
                int dCnt = Segments[j].NumberOfSamples;
                Segments[j].Waveform(DataSourceResultType.DataSourceResultType_Double64, 1, dCnt, 1, out Result);
                if (Result == null)
                    continue;
                double[] Buffer = (double[])Result;

                // sum all sample values
                for (int m = 0; m < Buffer.Length; m++)
                {
                    dAverage = dAverage + Buffer[m];
                }

                // sum the total number of samples
                iTotalCnt = iTotalCnt + dCnt;
            }
            if (iTotalCnt != 0)
            {
                // calculate the average
                dAverage = dAverage / iTotalCnt;
            }

            // loop through all returned segments to create a new waveform
            for (int i = 1; i <= Segments.Count; i++)
            {
                int dSampleCount = Segments[i].NumberOfSamples;
                Segments[i].Waveform(DataSourceResultType.DataSourceResultType_Double64, 1, dSampleCount, 1, out Result);
                double dXStep = Segments[i].SampleInterval;

                Block = MyWave.AddDataBlock(Segments[i].StartTime, dXStep);
                Block.WriteWaveform(0, (double[])Result);

                // create a block to waveform with average value
                BlockAve = MyWaveAve.AddDataBlock(Segments[i].StartTime, dXStep);
                double[] ResultAve = new double[dSampleCount];
                for (int k = 1; k <= dSampleCount; k++)
                {
                    ResultAve[k - 1] = Convert.ToSingle(dAverage);
                }
                BlockAve.WriteWaveform(0, ResultAve);
            }

            SegmentsCnt.Text = Segments.Count.ToString();

            display1.pDisplay.TimeDisplay.CtlLayout.Pages.ActivePage.Panes.AddPane().Activate();
            string[] aPoolEntry = new string[] { "CSIDemo.MyResult" };
            display1.pDisplay.AddDataSources(aPoolEntry);

            // add also - overlayed - the average value
            aPoolEntry = new string[] { "CSIDemo.MyResultAve" };
            display1.pDisplay.AddDataSources(aPoolEntry);
            display1.pDisplay.TimeDisplay.CtlLayout.ActiveTrace.TraceProp.PrimaryColor = 0x00FFFF; // yellow
        }

        private void btnCalc_Click(object sender, EventArgs e)
        {
            // Get Active Trace
            DTrace myTrace = display1.pDisplay.TimeDisplay.ActiveTrace;
            if (myTrace == null)
            {
                PerceptionMessageBox.Show(this, "No active trace found", "CSI Display Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            // Get data source from active trace
            IDataSrc iActiveTrace = myTrace.TraceProp.DataSrc;
            if (iActiveTrace == null)
            {
                PerceptionMessageBox.Show(this, "Active trace has no data source", "CSI Display Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Create a new Min Max calculator
            WavMinMax Calculator = new WavMinMax(iActiveTrace, double.MinValue, double.MaxValue);
            // Execute the calculator
            WavCalcResult CalcResult = Calculator.Exec();
            // Check if result is OK
            if (CalcResult != WavCalcResult.OK)
            {
                PerceptionMessageBox.Show(this, "Can not calculate Max/Min\r\n\r\n" + Calculator.ErrorCode, "CSI Display Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            // Get the results from the calculator
            double dMin;
            double dMax;
            double dMinPos;
            double dMaxPos;
            Calculator.Result(out dMin, out dMax, out dMinPos, out dMaxPos);

            // Show the results using Perception conversion functions
            lblMax.Text = FloatingPoint.ConvertDoubleToString(dMax, 4, iActiveTrace.YUnit) + iActiveTrace.YUnit;
            lblMin.Text = FloatingPoint.ConvertDoubleToString(dMin, 4, iActiveTrace.YUnit) + iActiveTrace.YUnit;

            lblMaxPos.Text = FloatingPoint.ConvertDoubleToString(dMaxPos, 4, iActiveTrace.YUnit) + iActiveTrace.XUnit;
            lblMinPos.Text = FloatingPoint.ConvertDoubleToString(dMinPos, 4, iActiveTrace.YUnit) + iActiveTrace.XUnit;

            // Position the vertical cursors at the maximum and minimum position
            display1.pDisplay.TimeDisplay.Cursors[1].time = dMaxPos;
            display1.pDisplay.TimeDisplay.Cursors[2].time = dMinPos;

            // Set the Horizontal cursors to the maximum and minimum levels.
            display1.pDisplay.TimeDisplay.CtlLayout.HorizontalCursors.Visible = true;

            // Get the high and low display range values from the active trace  
            double dHigh, dLow;
            myTrace.TraceProp.GetRange(out dHigh, out dLow);
            // Use these values to get the relative levels via a linear conversion
            double dLevel1 = (dMax - dLow) / (dHigh - dLow);
            double dLevel2 = (dMin - dLow) / (dHigh - dLow);

            // We also need to know the position of the active pane.
            double dTop = 1;
            double dBottom = 0;
            myTrace.Pane.GetPositions(ref dTop, ref dBottom);
            dLevel1 = (dTop - dBottom) * dLevel1 + dBottom;
            dLevel2 = (dTop - dBottom) * dLevel2 + dBottom;

            // Now we can set the relative level values
            display1.pDisplay.TimeDisplay.CtlLayout.HorizontalCursors[1].Location = dLevel1;
            display1.pDisplay.TimeDisplay.CtlLayout.HorizontalCursors[2].Location = dLevel2;

            // Before adding new markers clear all existing ones
            ClearAllMarkers();

            // Create a new display marker to indicate the maximum position
            DisplayMarker myTraceMarkerMax = myTrace.DisplayMarkers.AddMarker(DisplayModeType.DisplayModeType_Review);
            myTraceMarkerMax.DrawingType = MarkerDrawingType.MarkerDrawingType_TraceMarker;
            myTraceMarkerMax.MarkerStartSourceX = dMaxPos;
            myTraceMarkerMax.MarkerStartSourceY = dMax;
            myTraceMarkerMax.UnformattedText = "Max";
            myTraceMarkerMax.SetLabelPosition(DisplayModeType.DisplayModeType_Review, LabelAlignment.LabelAlignment_Left, -0.1, -0.1);

            // Create a new display marker to indicate the minimum position
            DisplayMarker myTraceMarkerMin = myTrace.DisplayMarkers.AddMarker(DisplayModeType.DisplayModeType_Review);
            myTraceMarkerMin.DrawingType = MarkerDrawingType.MarkerDrawingType_TraceMarker;
            myTraceMarkerMin.MarkerStartSourceX = dMinPos;
            myTraceMarkerMin.MarkerStartSourceY = dMin;
            myTraceMarkerMin.UnformattedText = "Min";
            myTraceMarkerMin.SetLabelPosition(DisplayModeType.DisplayModeType_Review, LabelAlignment.LabelAlignment_Left, 0.1, 0.1);
        }

        // Clears all existing markers
        private void ClearAllMarkers()
        {
            // get the number of display pages
            int NumberOfPages = display1.pDisplay.TimeDisplay.CtlLayout.Pages.Count;
            // Remove the markers per page
            for (int i = 1; i <= NumberOfPages; i++)
            {
                DPage myPage = display1.pDisplay.TimeDisplay.CtlLayout.Pages[i];
                // Get the display markers collection from the page
                DisplayMarkers myDisplayMarkers = myPage.DisplayMarkers;

                // Keep on deleting the markers until the collection is empty
                while (myDisplayMarkers.Count > 0)
                {
                    DisplayMarker myPageDisplayMarker = myDisplayMarkers[1];
                    myPageDisplayMarker.Delete();
                }

                // Now we will delete the display markers connected to the individual traces.
                // Therefore we have to loop through the panes 
                for (int n = 1; n <= myPage.Panes.Count; n++)
                {
                    DPane myPane = myPage.Panes[n];

                    // Go through all the traces from this pane
                    for (int m = 1; m <= myPane.Traces.Count; m++)
                    {
                        DTrace myTrace = myPane.Traces[m];
                        // Get the display markers collection from this trace
                        DisplayMarkers myTraceDisplayMarkers = myTrace.DisplayMarkers;

                        // Keep on deleting the markers until the collection is empty
                        while (myTraceDisplayMarkers.Count > 0)
                        {
                            DisplayMarker myTraceDisplayMarker = myTraceDisplayMarkers[1];
                            myTraceDisplayMarker.Delete();
                        }
                    }
                }
            }
        }

        private void btnCalcUsingForDB_Click(object sender, EventArgs e)
        {
            // Get Active Trace
            DTrace myTrace = display1.pDisplay.TimeDisplay.ActiveTrace;
            if (myTrace == null)
            {
                PerceptionMessageBox.Show(this, "No active trace found", "CSI Display Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            // Get data source from active trace
            IDataSrc iActiveTrace = myTrace.TraceProp.DataSrc;
            if (iActiveTrace == null)
            {
                PerceptionMessageBox.Show(this, "Active trace has no data source", "CSI Display Demo", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Get Data source name from trace e.g.: "Active.Group1.Recorder_B.Burst"
            string cDSName = myTrace.TraceProp.DataSourceName;

            // create required formulas in formula database

            // create instance of formula database
            FormulaDB ForDB = FormulaDB.Instance;

            // check if formula already exist
            if (ForDB.Formulas["CSI:MinVal"] == null)
            {
                // search for an empty line
                foreach (Formula frml in ForDB.Formulas)
                {
                    if (frml.IsEmpty())
                    {
                        // Define Name, Formula and Units (See columns formula database sheet)
                        frml.Name = "CSI:MinVal";
                        // Expression can be: "@Min(Active.Group1.Recorder_B.Burst)"
                        frml.Expression = string.Format("@Min({0})", cDSName);
                        ForDB.Formulas[frml.LineNumber + 1].Name = "CSI:MinPos";
                        // Expression can be: "@MinPos(Active.Group1.Recorder_B.Burst)"
                        ForDB.Formulas[frml.LineNumber + 1].Expression = string.Format("@MinPos({0})", cDSName);

                        // fetch units from source
                        m_MySource = m_DataManager.PoolEntries[cDSName].DataSource;
                        frml.Units = iActiveTrace.YUnit;
                        ForDB.Formulas[frml.LineNumber + 1].Units = iActiveTrace.XUnit;
                        break;
                    }
                }
            }

            // display formatted value
            m_MySource = m_DataManager.PoolEntries["Formula.CSI:MinVal"].DataSource;
            double dMinVal = (double)m_MySource.Value;
            lblMinForDB.Text = FloatingPoint.ConvertDoubleToString(dMinVal, 4, iActiveTrace.YUnit) + iActiveTrace.YUnit;

            // place cursor
            m_MySource = m_DataManager.PoolEntries["Formula.CSI:MinPos"].DataSource;
            if (m_MySource == null)
                return;
            double dMinPos = (double)m_MySource.Value;
            display1.pDisplay.TimeDisplay.Cursors[1].time = dMinPos;
            lblMinPosForDB.Text = FloatingPoint.ConvertDoubleToString(dMinPos, 4, iActiveTrace.XUnit) + iActiveTrace.XUnit;

            // once done: clean up the mess
            ForDB.Formulas["CSI:MinVal"].Clear();
            ForDB.Formulas["CSI:MinPos"].Clear();
        }


        #region Disposing
        /// <summary>
        /// Procedure to do your cleanup, this function is called from within the Dispose() method.
        /// </summary>
        private void SheetDisposed(object sender, EventArgs e)
        {
            if (IsDisposed)
                return;

            if (m_bDisposed)
                return;
            try
            {
                // Add your clean up code
                UnHookFromActiveCursor();
                m_ActiveCursor = null;
                m_DataManager = null;
                if (m_Channel != null)
                {
                    UnHookChannelEvent();
                    m_Channel = null;
                }
                UnHookFromGroupAll();
                m_GroupAll = null;
                m_MyDemoSystem = null;
            }
            catch
            {
            }
            m_bDisposed = true;
        }


        #endregion
    }
}